from .analytics import add_ranking_explanation, anisotropic_distance, deterministic_tie_break, directional_corridor_accessibility, filter_by_distance_cutoff, filter_by_threshold, jaccard_area_similarity, overlap_ratio_similarity
from .config import GeoPromptConfig, load_config
from .equations import (
    accessibility_potential,
    anisotropic_friction_cost,
    area_similarity,
    attraction_repulsion_score,
    acoustic_reverberation_index,
    balanced_opportunity_score,
    birth_rate_modifier,
    bayesian_update_weight,
    boundary_friction_score,
    business_cluster_advantage,
    calibration_error_score,
    capped_interaction,
    carbon_intensity_score,
    cauchy_decay,
    climate_vulnerability_index,
    coastal_inundation_risk,
    circularity_score,
    community_cohesion_score,
    competitive_influence,
    composite_suitability_score,
    congestion_penalized_interaction,
    consumer_purchasing_power,
    coordinate_distance,
    corridor_reliability_score,
    corridor_resilience_score,
    corridor_strength,
    cosine_taper_decay,
    coverage_equity_score,
    cultural_similarity_index,
    demand_supply_balance_score,
    density_similarity,
    digital_divide_index,
    directional_alignment,
    directional_bearing,
    directional_flow_score,
    drought_stress_index,
    emergency_response_score,
    energy_burden_score,
    entropy_evenness_index,
    entropy_similarity,
    euclidean_distance,
    exponential_decay,
    gaussian_decay,
    gentrification_pressure_index,
    geometric_interaction,
    gini_inequality_index,
    gompertz_decay,
    gravity_interaction,
    green_space_benefits_radius,
    hazard_exposure_score,
    habitat_fragmentation_score,
    harmonic_interaction,
    haversine_distance,
    hotspot_intensity_score,
    immunity_barrier_score,
    incidence_rate_decay,
    infrastructure_lifecycle_score,
    information_diffusion_rate,
    innovation_velocity,
    inverse_square_decay,
    land_value_gradient,
    linear_cutoff_decay,
    logistic_decay,
    market_concentration_index,
    fairness_parity_score,
    migration_attraction_index,
    mixed_use_score,
    mode_share_incentive,
    mortality_risk_index,
    multi_scale_accessibility_score,
    network_centrality_influence,
    network_redundancy_gain,
    noise_propagation_level,
    normalized_access_gap,
    opportunity_pressure_index,
    parking_availability_factor,
    productivity_efficiency,
    procurement_risk_score,
    path_detour_penalty,
    pollution_dispersion_model,
    population_carrying_capacity,
    prompt_decay,
    prompt_influence,
    prompt_interaction,
    rational_quadratic_decay,
    route_efficiency_score,
    rental_displacement_risk,
    renewable_penetration_score,
    school_access_score,
    scenario_robustness_score,
    service_reliability_decay,
    signal_to_noise_ratio_score,
    supply_chain_resilience,
    service_overlap_penalty,
    shape_compactness_similarity,
    sky_view_factor,
    softplus_decay,
    tanh_decay,
    temporal_stability_score,
    threshold_interaction,
    trend_momentum_score,
    trade_flow_intensity,
    transmission_risk_score,
    transit_accessibility_score,
    transit_support_score,
    vaccination_protection_score,
    visual_prominence_score,
    volatility_penalty_score,
    walkability_score,
    wildfire_risk_index,
    weibull_decay,
    queue_delay_factor,
    anomaly_severity_score,
    uncertainty_penalty,
    flood_storage_effectiveness,
    healthcare_access_index,
    food_desert_risk,
    heat_island_intensity,
    labor_market_tightness,
    evacuation_time_index,
    landslide_susceptibility,
    adaptive_capacity_score,
    fiscal_stability_index,
)
from .exceptions import AnchorError, CRSError, ColumnError, ConfigError, GeoPromptError, GeometryError, IOError_, PluginError, ValidationError
from .frame import Bounds, GeoPromptAnalysis, GeoPromptFrame
from .geometry import geometry_area, geometry_bounds, geometry_centroid, geometry_contains, geometry_distance, geometry_intersects, geometry_intersects_bounds, geometry_length, geometry_type, geometry_within, geometry_within_bounds, transform_geometry
from .io import frame_to_geojson, read_features, read_features_chunked, read_geojson, read_points, write_geojson
from .logging_ import configure_logging, log_timing, log_trace
from .normalization import normalize, normalize_min_max, normalize_robust, normalize_z_score
from .overlay import buffer_geometries, dissolve_geometries, geometry_from_shapely, geometry_to_geojson, geometry_to_shapely
from .plugins import get_decay, get_kernel, list_decay_functions, list_kernels, register_decay, register_kernel
from .sensitivity import SweepResult, confidence_score, parameter_sweep, rank_with_confidence
from .types import BoundsDict, DemoReport, InteractionRecord, NearestNeighborRecord
from .validation import SCHEMA_VERSION, add_schema_version, safe_weight, validate_crs, validate_distance_method_crs, validate_geometry, validate_non_empty_features, validate_numeric_range, validate_required_columns, validate_weight_column_values


API_STABILITY: dict[str, str] = {
    "GeoPromptFrame": "stable",
    "GeoPromptAnalysis": "stable",
    "read_features": "stable",
    "read_geojson": "stable",
    "write_geojson": "stable",
    "plugins": "beta",
    "sensitivity": "beta",
    "benchmarks": "experimental",
}


__all__ = [
    "AnchorError",
    "API_STABILITY",
    "Bounds",
    "BoundsDict",
    "CRSError",
    "ColumnError",
    "ConfigError",
    "DemoReport",
    "GeoPromptConfig",
    "GeoPromptError",
    "GeoPromptFrame",
    "GeoPromptAnalysis",
    "GeometryError",
    "IOError_",
    "InteractionRecord",
    "NearestNeighborRecord",
    "PluginError",
    "SCHEMA_VERSION",
    "SweepResult",
    "ValidationError",
    "accessibility_potential",
    "acoustic_reverberation_index",
    "add_ranking_explanation",
    "add_schema_version",
    "anisotropic_distance",
    "anisotropic_friction_cost",
    "area_similarity",
    "attraction_repulsion_score",
    "balanced_opportunity_score",
    "birth_rate_modifier",
    "bayesian_update_weight",
    "boundary_friction_score",
    "buffer_geometries",
    "business_cluster_advantage",
    "calibration_error_score",
    "capped_interaction",
    "carbon_intensity_score",
    "cauchy_decay",
    "coastal_inundation_risk",
    "circularity_score",
    "climate_vulnerability_index",
    "community_cohesion_score",
    "competitive_influence",
    "composite_suitability_score",
    "confidence_score",
    "configure_logging",
    "congestion_penalized_interaction",
    "consumer_purchasing_power",
    "coordinate_distance",
    "corridor_reliability_score",
    "corridor_resilience_score",
    "corridor_strength",
    "cosine_taper_decay",
    "coverage_equity_score",
    "cultural_similarity_index",
    "demand_supply_balance_score",
    "density_similarity",
    "digital_divide_index",
    "deterministic_tie_break",
    "directional_alignment",
    "directional_bearing",
    "directional_corridor_accessibility",
    "directional_flow_score",
    "dissolve_geometries",
    "entropy_similarity",
    "entropy_evenness_index",
    "euclidean_distance",
    "exponential_decay",
    "filter_by_distance_cutoff",
    "filter_by_threshold",
    "frame_to_geojson",
    "gaussian_decay",
    "gentrification_pressure_index",
    "geometric_interaction",
    "gini_inequality_index",
    "geometry_area",
    "geometry_bounds",
    "geometry_centroid",
    "geometry_contains",
    "geometry_distance",
    "geometry_from_shapely",
    "geometry_intersects",
    "geometry_intersects_bounds",
    "geometry_length",
    "geometry_to_geojson",
    "geometry_to_shapely",
    "geometry_type",
    "geometry_within",
    "geometry_within_bounds",
    "get_decay",
    "get_kernel",
    "gompertz_decay",
    "gravity_interaction",
    "green_space_benefits_radius",
    "hazard_exposure_score",
    "habitat_fragmentation_score",
    "harmonic_interaction",
    "haversine_distance",
    "hotspot_intensity_score",
    "immunity_barrier_score",
    "incidence_rate_decay",
    "infrastructure_lifecycle_score",
    "information_diffusion_rate",
    "innovation_velocity",
    "inverse_square_decay",
    "jaccard_area_similarity",
    "land_value_gradient",
    "linear_cutoff_decay",
    "list_decay_functions",
    "list_kernels",
    "load_config",
    "log_timing",
    "log_trace",
    "logistic_decay",
    "market_concentration_index",
    "fairness_parity_score",
    "migration_attraction_index",
    "mixed_use_score",
    "mode_share_incentive",
    "mortality_risk_index",
    "multi_scale_accessibility_score",
    "network_centrality_influence",
    "network_redundancy_gain",
    "noise_propagation_level",
    "normalized_access_gap",
    "normalize",
    "normalize_min_max",
    "normalize_robust",
    "normalize_z_score",
    "opportunity_pressure_index",
    "overlap_ratio_similarity",
    "parameter_sweep",
    "parking_availability_factor",
    "productivity_efficiency",
    "procurement_risk_score",
    "path_detour_penalty",
    "pollution_dispersion_model",
    "population_carrying_capacity",
    "prompt_decay",
    "prompt_influence",
    "prompt_interaction",
    "rank_with_confidence",
    "rational_quadratic_decay",
    "read_features",
    "read_geojson",
    "read_points",
    "register_decay",
    "register_kernel",
    "route_efficiency_score",
    "rental_displacement_risk",
    "renewable_penetration_score",
    "school_access_score",
    "scenario_robustness_score",
    "service_reliability_decay",
    "signal_to_noise_ratio_score",
    "supply_chain_resilience",
    "safe_weight",
    "service_overlap_penalty",
    "shape_compactness_similarity",
    "sky_view_factor",
    "softplus_decay",
    "tanh_decay",
    "temporal_stability_score",
    "threshold_interaction",
    "trend_momentum_score",
    "trade_flow_intensity",
    "transform_geometry",
    "transmission_risk_score",
    "transit_accessibility_score",
    "transit_support_score",
    "vaccination_protection_score",
    "validate_crs",
    "validate_distance_method_crs",
    "validate_geometry",
    "validate_non_empty_features",
    "validate_numeric_range",
    "validate_required_columns",
    "validate_weight_column_values",
    "visual_prominence_score",
    "volatility_penalty_score",
    "walkability_score",
    "wildfire_risk_index",
    "queue_delay_factor",
    "anomaly_severity_score",
    "uncertainty_penalty",
    "flood_storage_effectiveness",
    "healthcare_access_index",
    "food_desert_risk",
    "heat_island_intensity",
    "labor_market_tightness",
    "evacuation_time_index",
    "landslide_susceptibility",
    "adaptive_capacity_score",
    "fiscal_stability_index",
    "weibull_decay",
    "write_geojson",
]